//
//  ViewController.m
//  Heap
//
//  Created by Grassi, Dan on 8/22/12.
//  Copyright (c) 2012 Zaph, llc. All rights reserved.
//

#import "ViewController.h"
#import "TreeDataModel.h"
#import "TreeView.h"
#import "Heap.h"

const NSTimeInterval kSwapDelay = 1.0;

@interface ViewController ()
@property (weak, nonatomic) IBOutlet TreeView *treeView;
@property (weak, nonatomic) IBOutlet UILabel *arrayLabel;
@property (weak, nonatomic) IBOutlet UIButton *addButton;
@property (weak, nonatomic) IBOutlet UIButton *deleteButton;
@property (weak, nonatomic) IBOutlet UISwitch *delaySwitch;

@property (strong, nonatomic) TreeDataModel *treeDataModel;
@property (strong, nonatomic) Node          *lastNode;
@property (strong, nonatomic) Heap          *heap;
@property (assign, nonatomic) int            random;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.treeDataModel = [TreeDataModel new];
    self.heap = [Heap new];
    self.lastNode = nil;
}

- (void)viewDidAppear:(BOOL)animated {
    [[NSNotificationCenter defaultCenter] addObserverForName:@"updateTreeView"
                                                      object:nil
                                                       queue:[NSOperationQueue mainQueue]
                                                  usingBlock:^(NSNotification *notification) {
                                                      [self refresh];
                                                  }];

    self.treeView.vertexSize = 10;
    self.treeView.lineWidth  = 1.5;
    self.treeView.fontSize   = 12;
    self.treeView.treeDataModel = self.treeDataModel;

    [self setupAddButton];
    [self setupDeleteButton];
    [self refresh];
}

- (void)setupAddButton {
    self.random = arc4random() % 100;
    NSString *title = [NSString stringWithFormat:@"Add %i", self.random];
    [self.addButton setTitle:title forState:UIControlStateNormal];
}

- (void)setupDeleteButton {
    HeapItem *lastItem = [self.heap topObject];
    NSString *title = @"";
    if (lastItem) {
        int value = lastItem.value;
        title = [NSString stringWithFormat:@"Delete %i", value];
    }
    [self.deleteButton setTitle:title forState:UIControlStateNormal];
}


- (Node *)treeForHeap {
    HeapItem *heapTop  = [self.heap topObject];
    if (heapTop == 0)
        return nil;

    Node *placeHolderNode = [Node new];
    [self addNodesToNode:placeHolderNode treeForHeapParent:heapTop];
    return [placeHolderNode childAtIndex:0];
}

- (Node *)addNodesToNode:(Node *)node treeForHeapParent:(HeapItem *)parent {
    NSString *name = [NSString stringWithFormat:@"%i", parent.value];
    Node *newNode = [Node nodeWithText:name];
    newNode.value = parent.itemID;
    [node addChild:newNode];

    HeapItem *leftChild  = [self.heap leftChildForParent: parent];
    if (leftChild)
        [self addNodesToNode:newNode treeForHeapParent:leftChild];

    HeapItem * rightChild = [self.heap rightChildForParent:parent];
    if (rightChild)
        [self addNodesToNode:newNode treeForHeapParent:rightChild];

    return node;
}

- (void)refresh {
    self.arrayLabel.attributedText = [self.heap attributedDescriptionByValue];

    self.treeDataModel.topNode =  [self treeForHeap];
    [self.treeDataModel updateLevels];
    [self.treeView setNeedsDisplay];
}


- (IBAction)addButton:(UIButton *)sender {
    sender.enabled = NO;
    self.treeView.highlightID = self.heap.nextItemID;

    [self.heap insertItemWithValue:self.random];

    self.treeView.highlightID = -1;
    [self refresh];
    [self setupAddButton];
    sender.enabled = YES;

    [self setupDeleteButton];
}

- (IBAction)deleteButton:(UIButton *)sender {
    sender.enabled = NO;
    HeapItem *lastItem = [self.heap lastObject];
    self.treeView.highlightID = lastItem.itemID;

    [self.heap deleteItem];

    self.treeView.highlightID = -1;
    [self refresh];
    sender.enabled = YES;

    [self setupDeleteButton];
}

- (IBAction)delaySwitch:(UISwitch *)sender {
    self.heap.delay = (sender.on) ? kSwapDelay : 0.0;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [self refresh];
}

@end
