//
//  Node.m
//  Tree
//
//  Created by Grassi, Dan on 8/14/12.
//  Copyright (c) 2012 Zaph, llc. All rights reserved.
//

#import "Node.h"

static int globalNodeID = 0;


@interface Node ()
@property (strong, nonatomic) NSMutableArray *children;
@end

@implementation Node

- (id)init {
    return [self initWithText:nil];
}

- (id)initWithText:(NSString *)text {
    self = [super init];
    if (self) {
        _nodeID   = ++globalNodeID;
        _level    = 0;
        _location = 0;
        _width    = 1;
        _text     = text;
        _parent   = nil;
        _children = [NSMutableArray array];
        if (text == nil)
            _text = [NSString stringWithFormat:@"%i", _nodeID];
    }
    return self;
}

+ (id)nodeWithText:(NSString *)text {
    return [[Node alloc] initWithText:text];
}

- (NSUInteger)countByEnumeratingWithState:(NSFastEnumerationState *)state objects:(id __unsafe_unretained [])stackbuf count:(NSUInteger)len {
    return [self.children countByEnumeratingWithState:state objects:stackbuf count:len];
}

- (NSString *)description {
    NSMutableString *text = [NSMutableString string];
    [text appendString:@"\nNode:"];
    [text appendFormat:@"\nnodeID:   %i", self.nodeID];
    [text appendFormat:@"\nlocation: %i", self.location];
    [text appendFormat:@"\ntext:     %@", self.text];
    return text;
}

- (NSUInteger)childCount {
    return self.children.count;
}

- (void)addChild:(Node *)child {
    [self.children addObject:child];
    child.parent = self;
}

- (Node *)childAtIndex:(int)index {
    Node *child = nil;
    if (self.children.count) {
        child = self.children[index];
    }
    return child;
}

- (Node *)lastChild {
    return self.children[self.children.count-1];
}

@end
