//
//  TreeDataModel.m
//  Tree
//
//  Created by Grassi, Dan on 8/14/12.
//  Copyright (c) 2012 Zaph, llc. All rights reserved.
//


#import "TreeDataModel.h"

@interface TreeDataModel ()
@property (assign, readwrite, nonatomic) int depth;
@property (assign, nonatomic) int maxLevel;
@property (strong, nonatomic) Node *rightMost;
@property (strong, nonatomic) Node *leftMost;
@end

@implementation TreeDataModel

- (id)init {
    self = [super init];
    if (self) {
        _maxLevel = 0;
        _depth = 0;
    }
    return self;
}

- (void)updateLevels {
    self.topNode.level = 0;
    self.topNode.location = 0;
    self.rightMost = self.topNode;
    self.leftMost = self.topNode;
    self.maxLevel = 0;

    [self updateLevelsforNode:self.topNode];
    self.depth = self.maxLevel;
}

- (void)updateLevels:(Node *)node {
    self.topNode = node;
    [self updateLevels];
}

- (void)updateLevelsforNode:(Node *)node {
    node.width = 1;
    int location = 0;
    int childrenWidth = 0;
    for (Node *subNode in node) {
        subNode.location = location++;
        subNode.level = node.level+1;
        [self updateLevelsforNode:subNode];
        childrenWidth += subNode.width;
    }
    node.width = MAX(node.width, childrenWidth);
    self.maxLevel = MAX(self.maxLevel, node.level);
}

- (Node *)findNodeID:(int)nodeID {
    return [self fineNodeID:nodeID inTreeNode:self.topNode];
}

- (Node *)fineNodeID:(int)nodeID inTreeNode:(Node *)node {
    if (node.nodeID == nodeID)
        return node;

    if (node.childCount == 0)
        return nil;
    Node *foundNode = nil;
    for (Node *subNode in node) {
        foundNode = [self fineNodeID:nodeID inTreeNode:subNode];
        if (foundNode)
            break;
    }
    return foundNode;
}

- (void)reportLevelsforNode:(Node *)node {
    NSLog(@"nodeID:%2i, level:%i, location:%i, width:%i", node.nodeID, node.level, node.location, node.width);
    for (Node *subNode in node) {
        [self reportLevelsforNode:subNode];
    }
}

@end
